@@ -258,8 +258,6 @@ class BaseRoute(Transactional):
                 return
 
             self['ipdb_scope'] = 'system'
-            for (key, value) in msg.items():
-                self[key] = value
 
             # IPv6 multipath via several devices (not networks) is a very
             # special case, since we get only the first hop notification. Ask
@@ -269,6 +267,8 @@ class BaseRoute(Transactional):
             flags = msg.get('header', {}).get('flags', 0)
             family = msg.get('family', 0)
             clean_mp = True
+            table = msg.get_attr('RTA_TABLE') or msg.get('table')
+            dst = msg.get_attr('RTA_DST')
             #
             # It MAY be a multipath hop
             #
@@ -284,9 +284,9 @@ class BaseRoute(Transactional):
                     #
                     clean_mp = False
                     msgs = self.nl.route('show',
-                                         table=self['table'],
-                                         dst=self['dst'],
-                                         family=self['family'])
+                                         table=table,
+                                         dst=dst,
+                                         family=family)
                     for nhmsg in msgs:
                         nh = type(self)(ipdb=self.ipdb, parent=self)
                         nh.load_netlink(nhmsg)
@@ -303,8 +303,18 @@ class BaseRoute(Transactional):
                 elif flags == NLM_F_MULTI and self.get('dst'):
                     nh = type(self)(ipdb=self.ipdb, parent=self)
                     nh.load_netlink(msg)
+                    with nh._direct_state:
+                        del nh['dst']
+                        del nh['ipdb_scope']
+                        del nh['ipdb_priority']
+                        del nh['multipath']
+                        del nh['metrics']
+                    self.add_nh(nh)
                     return
 
+            for (key, value) in msg.items():
+                self[key] = value
+
             # cleanup multipath NH
             if clean_mp:
                 for nh in self['multipath']:
@@ -342,8 +352,8 @@ class BaseRoute(Transactional):
                         ret = []
                         # FIXME: should support encap_types other than MPLS
                         try:
-                            for l in value.get_attr('MPLS_IPTUNNEL_DST'):
-                                ret.append(str(l['label']))
+                            for dst in value.get_attr('MPLS_IPTUNNEL_DST'):
+                                ret.append(str(dst['label']))
                             if ret:
                                 self['encap']['labels'] = '/'.join(ret)
                         except AttributeError:
@@ -866,7 +876,8 @@ class RoutingTable(object):
             if now - route['route']._gctime < 2:
                 continue
             try:
-                self.ipdb.nl.route('get', **route['route'])
+                if not self.ipdb.nl.route('dump', **route['route']):
+                    raise
                 with route['route']._direct_state:
                     route['route']['ipdb_scope'] = 'system'
             except:
@@ -1028,15 +1039,19 @@ class RoutingTableSet(object):
         self.tables = {254: RoutingTable(self.ipdb)}
         self._event_map = {'RTM_NEWROUTE': self.load_netlink,
                            'RTM_DELROUTE': self.load_netlink,
+                           'RTM_NEWLINK': self.gc_mark_link,
                            'RTM_DELLINK': self.gc_mark_link,
                            'RTM_DELADDR': self.gc_mark_addr}
 
     def _register(self):
-        for msg in self.ipdb.nl.get_routes(family=AF_INET):
+        for msg in self.ipdb.nl.get_routes(family=AF_INET,
+                                           match={'family': AF_INET}):
             self.load_netlink(msg)
-        for msg in self.ipdb.nl.get_routes(family=AF_INET6):
+        for msg in self.ipdb.nl.get_routes(family=AF_INET6,
+                                           match={'family': AF_INET6}):
             self.load_netlink(msg)
-        for msg in self.ipdb.nl.get_routes(family=AF_MPLS):
+        for msg in self.ipdb.nl.get_routes(family=AF_MPLS,
+                                           match={'family': AF_MPLS}):
             self.load_netlink(msg)
 
     def add(self, spec=None, **kwarg):
@@ -1059,6 +1074,13 @@ class RoutingTableSet(object):
                 spec['family'] = AF_INET
         if not dst:
             raise ValueError('dst not specified')
+        if isinstance(dst, basestring) and \
+                (dst not in ('', 'default')) and \
+                ('/' not in dst):
+            if spec['family'] == AF_INET:
+                spec['dst'] = dst + '/32'
+            elif spec['family'] == AF_INET6:
+                spec['dst'] = dst + '/128'
         if 'priority' not in spec:
             if spec['family'] == AF_INET6:
                 spec['priority'] = IP6_RT_PRIO_USER
@@ -1200,7 +1222,7 @@ class RoutingTableSet(object):
         ###
         # mark route records for GC after link delete
         #
-        if msg['family'] != 0:
+        if msg['family'] != 0 or msg['state'] != 'down':
             return
 
         for record in self.filter({'oif': msg['index']}):
