@@ -44,10 +44,12 @@ On all the supported platforms, be it Linux or BSD, the
 payload and `IPRoute.get(...)` returns parsed RTNL messages.
 '''
 import os
+import errno
 import struct
 import select
 import threading
 
+from pyroute2 import config
 from pyroute2.netlink import (NLM_F_REQUEST,
                               NLM_F_DUMP,
                               NLM_F_MULTI,
@@ -89,6 +91,7 @@ class IPRoute(object):
             self._ssh = ['ssh', kwarg.pop('ssh')]
         else:
             self._ssh = []
+        async_qsize = kwarg.get('async_qsize')
         self._ifc = Ifconfig(cmd=self._ssh + ['ifconfig', '-a'])
         self._arp = ARP(cmd=self._ssh + ['arp', '-an'])
         self._route = Route(cmd=self._ssh + ['netstat', '-rn'])
@@ -101,7 +104,7 @@ class IPRoute(object):
         self._brd_socket = None
         self._pfdr, self._pfdw = os.pipe()  # notify external poll/select
         self._ctlr, self._ctlw = os.pipe()  # notify monitoring thread
-        self._outq = queue.Queue()
+        self._outq = queue.Queue(maxsize=async_qsize or config.async_qsize)
         self._system_lock = threading.Lock()
         self.closed = threading.Event()
 
@@ -114,7 +117,7 @@ class IPRoute(object):
     def clone(self):
         return self
 
-    def close(self):
+    def close(self, code=errno.ECONNRESET):
         with self._system_lock:
             if self.closed.is_set():
                 return
@@ -124,7 +127,8 @@ class IPRoute(object):
                 self._mon_th.join()
                 self._rtm.close()
 
-            self._outq.put(struct.pack('IHHQIQQ', 28, 2, 0, 0, 104, 0, 0))
+            if code > 0:
+                self._outq.put(struct.pack('IHHQIQQ', 28, 2, 0, 0, code, 0, 0))
             os.write(self._pfdw, b'\0')
             for ep in (self._pfdr, self._pfdw, self._ctlr, self._ctlw):
                 try:
@@ -239,6 +243,20 @@ class IPRoute(object):
         self._outq.put(data)
         os.write(self._pfdw, b'\0')
 
+    # 8<---------------------------------------------------------------
+    #
+    def dump(self):
+        '''
+        Iterate all the objects -- links, routes, addresses etc.
+        '''
+        for method in (self.get_links,
+                       self.get_addr,
+                       self.get_neighbours,
+                       self.get_routes):
+            for msg in method():
+                yield msg
+    # 8<---------------------------------------------------------------
+
     def get_links(self, *argv, **kwarg):
         ret = []
         data = self._ifc.run()
