@@ -20,9 +20,8 @@ Get interface objects
 
 The keys may be used as selectors to get interface objects::
 
-    with NDB(log='on') as ndb:
-        for key in ndb.interfaces:
-            print(ndb.interfaces[key])
+    for key in ndb.interfaces:
+        print(ndb.interfaces[key])
 
 Also possible selector formats are `dict()` and simple string. The latter
 means the interface name::
@@ -42,44 +41,38 @@ Change nic properties
 
 Changing MTU and MAC address::
 
-    with NDB(log='on') as ndb:
-        with ndb.interfaces['eth0'] as eth0:
-            eth0['mtu'] = 1248
-            eth0['address'] = '00:11:22:33:44:55'
-        # --> <-- eth0.commit() is called by the context manager
-    # --> <-- ndb.close() is called by the context manager
+    with ndb.interfaces['eth0'] as eth0:
+        eth0['mtu'] = 1248
+        eth0['address'] = '00:11:22:33:44:55'
+    # --> <-- eth0.commit() is called by the context manager
 
 One can change a property either using the assignment statement, or
 using the `.set()` routine::
 
     # same code
-    with NDB(log='on') as ndb:
-        with ndb.interfaces['eth0'] as eth0:
-            eth0.set('mtu', 1248)
-            eth0.set('address', '00:11:22:33:44:55')
+    with ndb.interfaces['eth0'] as eth0:
+        eth0.set('mtu', 1248)
+        eth0.set('address', '00:11:22:33:44:55')
 
 The `.set()` routine returns the object itself, that makes possible
 chain calls::
 
     # same as above
-    with NDB(log='on') as ndb:
-        with ndb.interfaces['eth0'] as eth0:
-            eth0.set('mtu', 1248).set('address', '00:11:22:33:44:55')
+    with ndb.interfaces['eth0'] as eth0:
+        eth0.set('mtu', 1248).set('address', '00:11:22:33:44:55')
 
     # or
-    with NDB(log='on') as ndb:
-        with ndb.interfaces['eth0'] as eth0:
-            (eth0
-             .set('mtu', 1248)
-             .set('address', '00:11:22:33:44:55'))
+    with ndb.interfaces['eth0'] as eth0:
+        (eth0
+         .set('mtu', 1248)
+         .set('address', '00:11:22:33:44:55'))
 
     # or without the context manager, call commit() explicitly
-    with NDB(log='on') as ndb:
-        (ndb
-         .interfaces['eth0']
-         .set('mtu', 1248)
-         .set('address', '00:11:22:33:44:55')
-         .commit())
+    (ndb
+     .interfaces['eth0']
+     .set('mtu', 1248)
+     .set('address', '00:11:22:33:44:55')
+     .commit())
 
 Create virtual interfaces
 =========================
@@ -98,15 +91,15 @@ Create a bridge and add a port, `eth0`::
 
 '''
 
-import weakref
 import traceback
-from collections import OrderedDict
 from pyroute2.config import AF_BRIDGE
 from pyroute2.ndb.objects import RTNL_Object
 from pyroute2.common import basestring
 from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg
 from pyroute2.netlink.rtnl.p2pmsg import p2pmsg
 from pyroute2.netlink.exceptions import NetlinkError
+from pyroute2.ndb.auth_manager import AuthManager
+from pyroute2.ndb.auth_manager import check_auth
 
 
 def load_ifinfmsg(schema, target, event):
@@ -129,8 +122,23 @@ def load_ifinfmsg(schema, target, event):
     #
     if event['family'] == AF_BRIDGE:
         #
-        # bypass for now
-        #
+        schema.load_netlink('af_bridge_ifs', target, event)
+        vlans = (event
+                 .get_attr('IFLA_AF_SPEC')
+                 .get_attrs('IFLA_BRIDGE_VLAN_INFO'))
+        # flush the old vlans info
+        schema.execute('''
+                       DELETE FROM af_bridge_vlans
+                       WHERE
+                           f_target = %s
+                           AND f_index = %s
+                       ''' % (schema.plch, schema.plch),
+                       (target, event['index']))
+        for v in vlans:
+            v['index'] = event['index']
+            v['header'] = {'type': event['header']['type']}
+            schema.load_netlink('af_bridge_vlans', target, v)
+
         return
 
     schema.load_netlink('interfaces', target, event)
@@ -173,19 +181,42 @@ def load_ifinfmsg(schema, target, event):
                 schema.load_netlink(table, target, ifdata)
 
 
-init = {'specs': [['interfaces', OrderedDict(ifinfmsg.sql_schema())],
-                  ['p2p', OrderedDict(p2pmsg.sql_schema())]],
+schema_ifinfmsg = (ifinfmsg
+                   .sql_schema()
+                   .unique_index('index'))
+
+schema_brinfmsg = (ifinfmsg
+                   .sql_schema()
+                   .unique_index('index')
+                   .foreign_key('interface',
+                                ('f_target', 'f_tflags', 'f_index'),
+                                ('f_target', 'f_tflags', 'f_index')))
+
+schema_p2pmsg = (p2pmsg
+                 .sql_schema()
+                 .unique_index('index')
+                 .foreign_key('interfaces',
+                              ('f_target', 'f_tflags', 'f_index'),
+                              ('f_target', 'f_tflags', 'f_index')))
+
+schema_af_bridge_vlans = (ifinfmsg
+                          .af_spec_bridge
+                          .vlan_info
+                          .sql_schema()
+                          .push('index', 'INTEGER')
+                          .unique_index('vid', 'index')
+                          .foreign_key('af_bridge_ifs',
+                                       ('f_target', 'f_tflags', 'f_index'),
+                                       ('f_target', 'f_tflags', 'f_index')))
+
+init = {'specs': [['interfaces', schema_ifinfmsg],
+                  ['af_bridge_ifs', schema_ifinfmsg],
+                  ['af_bridge_vlans', schema_af_bridge_vlans],
+                  ['p2p', schema_p2pmsg]],
         'classes': [['interfaces', ifinfmsg],
+                    ['af_bridge_ifs', ifinfmsg],
+                    ['af_bridge_vlans', ifinfmsg.af_spec_bridge.vlan_info],
                     ['p2p', p2pmsg]],
-        'indices': [['interfaces', ('index', )],
-                    ['p2p', ('index', )]],
-        'foreign_keys': [['p2p', [{'fields': ('f_target',
-                                              'f_tflags',
-                                              'f_index'),
-                                   'parent_fields': ('f_target',
-                                                     'f_tflags',
-                                                     'f_index'),
-                                   'parent': 'interfaces'}]]],
         'event_map': {ifinfmsg: [load_ifinfmsg]}}
 
 ifinfo_names = ('bridge',
@@ -202,29 +233,15 @@ supported_ifinfo = {x: ifinfmsg.ifinfo.data_map[x] for x in ifinfo_names}
 #
 for (name, data) in supported_ifinfo.items():
     name = 'ifinfo_%s' % name
-    #
-    # classes
-    #
     init['classes'].append([name, data])
-    #
-    # indices
-    #
-    init['indices'].append([name, ('index', )])
-    #
-    # spec
-    #
-    init['specs'].append([name, OrderedDict(data.sql_schema() +
-                                            [(('index', ), 'BIGINT')])])
-    #
-    # foreign keys
-    #
-    init['foreign_keys'].append([name, [{'fields': ('f_target',
-                                                    'f_tflags',
-                                                    'f_index'),
-                                         'parent_fields': ('f_target',
-                                                           'f_tflags',
-                                                           'f_index'),
-                                         'parent': 'interfaces'}]])
+    schema = (data
+              .sql_schema()
+              .push('index', 'BIGINT')
+              .unique_index('index')
+              .foreign_key('interfaces',
+                           ('f_target', 'f_tflags', 'f_index'),
+                           ('f_target', 'f_tflags', 'f_index')))
+    init['specs'].append([name, schema])
 
 
 def _cmp_master(self, value):
@@ -236,29 +253,107 @@ def _cmp_master(self, value):
     return False
 
 
+class Vlan(RTNL_Object):
+
+    table = 'af_bridge_vlans'
+    msg_class = ifinfmsg.af_spec_bridge.vlan_info
+    api = 'vlan_filter'
+
+    @classmethod
+    def _dump_where(cls, view):
+        if view.chain:
+            plch = view.ndb.schema.plch
+            where = '''
+                    WHERE
+                        main.f_target = %s AND
+                        main.f_index = %s
+                    ''' % (plch, plch)
+            values = [view.chain['target'], view.chain['index']]
+        else:
+            where = ''
+            values = []
+        return (where, values)
+
+    @classmethod
+    def summary(cls, view):
+        req = '''
+              SELECT
+                  main.f_target, main.f_tflags, main.f_vid,
+                  intf.f_IFLA_IFNAME
+              FROM
+                  af_bridge_vlans AS main
+              INNER JOIN
+                  interfaces AS intf
+              ON
+                  main.f_index = intf.f_index
+                  AND main.f_target = intf.f_target
+              '''
+        yield ('target', 'tflags', 'vid', 'ifname')
+        where, values = cls._dump_where(view)
+        for record in view.ndb.schema.fetch(req + where, values):
+            yield record
+
+    def __init__(self, *argv, **kwarg):
+        kwarg['iclass'] = ifinfmsg.af_spec_bridge.vlan_info
+        if 'auth_managers' not in kwarg or kwarg['auth_managers'] is None:
+            kwarg['auth_managers'] = []
+        log = argv[0].ndb.log.channel('vlan auth')
+        kwarg['auth_managers'].append(AuthManager({'obj:modify': False}, log))
+        super(Vlan, self).__init__(*argv, **kwarg)
+
+    def make_req(self, prime):
+        ret = {}
+        if 'index' in self:
+            ret['index'] = self['index']
+        ret['vlan_info'] = {'vid': self['vid']}
+        if 'flags' in self:
+            ret['vlan_info']['flags'] = self['flags']
+        return ret
+
+    def make_idx_req(self, prime):
+        return self.make_req(prime)
+
+
 class Interface(RTNL_Object):
 
     table = 'interfaces'
     msg_class = ifinfmsg
     api = 'link'
     key_extra_fields = ['IFLA_IFNAME']
-    summary = '''
-              SELECT
-                  a.f_target, a.f_tflags, a.f_index, a.f_IFLA_IFNAME,
-                  a.f_IFLA_ADDRESS, a.f_flags, a.f_IFLA_INFO_KIND
-              FROM
-                  interfaces AS a
-              '''
-    table_alias = 'a'
-    summary_header = ('target',
-                      'tflags',
-                      'index',
-                      'ifname',
-                      'lladdr',
-                      'flags',
-                      'kind')
     fields_cmp = {'master': _cmp_master}
 
+    @classmethod
+    def _dump_where(cls, view):
+        if view.chain:
+            plch = view.ndb.schema.plch
+            where = '''
+                    WHERE
+                        f_target = %s AND
+                        f_IFLA_MASTER = %s
+                    ''' % (plch, plch)
+            values = [view.chain['target'], view.chain['index']]
+        else:
+            where = ''
+            values = []
+        return (where, values)
+
+    @classmethod
+    def summary(cls, view):
+        req = '''
+              SELECT
+                  f_target, f_tflags, f_index,
+                  f_IFLA_IFNAME, f_IFLA_ADDRESS,
+                  f_flags, f_IFLA_INFO_KIND
+              FROM
+                  interfaces
+              '''
+        yield ('target', 'tflags', 'index',
+               'ifname', 'lladdr',
+               'flags', 'kind')
+        where, values = cls._dump_where(view)
+        for record in view.ndb.schema.fetch(req + where, values):
+            yield record
+
     def mark_tflags(self, mark):
         plch = (self.schema.plch, ) * 3
         self.schema.execute('''
@@ -279,60 +374,66 @@ class Interface(RTNL_Object):
 
     @property
     def ipaddr(self):
-        return (self
-                .view
-                .ndb
-                ._get_view('addresses',
-                           chain=self,
-                           match_src=[weakref.proxy(self),
-                                      {'index':
-                                       self.get('index', 0),
-                                       'target': self['target']}],
-                           match_pairs={'index': 'index',
-                                        'target': 'target'}))
+        return self.view.ndb._get_view('addresses', chain=self)
 
     @property
     def ports(self):
-        return (self
-                .view
-                .ndb
-                ._get_view('interfaces',
-                           chain=self,
-                           match_src=[weakref.proxy(self),
-                                      {'index':
-                                       self.get('index', 0),
-                                       'target': self['target']}],
-                           match_pairs={'master': 'index',
-                                        'target': 'target'}))
+        return self.view.ndb._get_view('interfaces', chain=self)
 
     @property
     def routes(self):
-        return (self
-                .view
-                .ndb
-                ._get_view('routes',
-                           chain=self,
-                           match_src=[weakref.proxy(self),
-                                      {'index':
-                                       self.get('index', 0),
-                                       'target': self['target']}],
-                           match_pairs={'oif': 'index',
-                                        'target': 'target'}))
+        return self.view.ndb._get_view('routes', chain=self)
 
     @property
     def neighbours(self):
-        return (self
-                .view
-                .ndb
-                ._get_view('neighbours',
-                           chain=self,
-                           match_src=[weakref.proxy(self),
-                                      {'index':
-                                       self.get('index', 0),
-                                       'target': self['target']}],
-                           match_pairs={'ifindex': 'index',
-                                        'target': 'target'}))
+        return self.view.ndb._get_view('neighbours', chain=self)
 
+    @property
+    def vlans(self):
+        return self.view.ndb._get_view('vlans', chain=self)
+
+    @property
+    def context(self):
+        ctx = {}
+        if self.get('target'):
+            ctx['target'] = self['target']
+        if self.get('index'):
+            ctx['index'] = self['index']
+        return ctx
+
+    @classmethod
+    def adjust_spec(cls, spec, context):
+        if isinstance(spec, basestring):
+            ret = {'ifname': spec}
+        else:
+            ret = dict(spec)
+        ret.update(context)
+        return ret
+
+    @check_auth('obj:modify')
+    def add_vlan(self, spec):
+        def do_add_vlan(self, spec):
+            try:
+                self.vlan.create(spec).apply()
+            except Exception as e_s:
+                e_s.trace = traceback.format_stack()
+                return e_s
+        self._apply_script.append((do_add_vlan, (self, spec), {}))
+        return self
+
+    @check_auth('obj:modify')
+    def del_vlan(self, spec):
+        def do_del_vlan(self, spec):
+            try:
+                ret = self.vlan[spec].remove().apply()
+            except Exception as e_s:
+                e_s.trace = traceback.format_stack()
+                return e_s
+            return ret.last_save
+        self._apply_script.append((do_del_vlan, (self, spec), {}))
+        return self
+
+    @check_auth('obj:modify')
     def add_ip(self, spec):
         def do_add_ip(self, spec):
             try:
@@ -343,6 +444,7 @@ class Interface(RTNL_Object):
         self._apply_script.append((do_add_ip, (self, spec), {}))
         return self
 
+    @check_auth('obj:modify')
     def del_ip(self, spec):
         def do_del_ip(self, spec):
             try:
@@ -354,6 +456,7 @@ class Interface(RTNL_Object):
         self._apply_script.append((do_del_ip, (self, spec), {}))
         return self
 
+    @check_auth('obj:modify')
     def add_port(self, spec):
         def do_add_port(self, spec):
             try:
@@ -368,6 +471,7 @@ class Interface(RTNL_Object):
         self._apply_script.append((do_add_port, (self, spec), {}))
         return self
 
+    @check_auth('obj:modify')
     def del_port(self, spec):
         def do_del_port(self, spec):
             try:
@@ -383,6 +487,7 @@ class Interface(RTNL_Object):
         self._apply_script.append((do_del_port, (self, spec), {}))
         return self
 
+    @check_auth('obj:modify')
     def __setitem__(self, key, value):
         if key == 'peer':
             dict.__setitem__(self, key, value)
@@ -419,7 +524,9 @@ class Interface(RTNL_Object):
                      .interfaces
                      .getmany({'IFLA_MASTER': self['index']})):
             # bridge ports
-            link = type(self)(self.view, spec)
+            link = type(self)(self.view,
+                              spec,
+                              auth_managers=self.auth_managers)
             snp.snapshot_deps.append((link, link.snapshot()))
         for spec in (self
                      .ndb
@@ -427,7 +534,9 @@ class Interface(RTNL_Object):
                      .getmany({'IFLA_LINK': self['index']})):
             # vlans & veth
             if self.get('link') != spec['index']:
-                link = type(self)(self.view, spec)
+                link = type(self)(self.view,
+                                  spec,
+                                  auth_managers=self.auth_managers)
                 snp.snapshot_deps.append((link, link.snapshot()))
         # return the root node
         return snp
@@ -438,9 +547,10 @@ class Interface(RTNL_Object):
             req['master'] = self['master']
         return req
 
+    @check_auth('obj:modify')
     def apply(self, rollback=False, fallback=False):
         # translate string link references into numbers
-        for key in ('link', ):
+        for key in ('link', 'master'):
             if key in self and isinstance(self[key], basestring):
                 self[key] = self.ndb.interfaces[self[key]]['index']
         setns = self.state.get() == 'setns'
@@ -453,7 +563,9 @@ class Interface(RTNL_Object):
                 key = dict(self)
                 key['create'] = True
                 del key['master']
-                fb = type(self)(self.view, key)
+                fb = type(self)(self.view,
+                                key,
+                                auth_managers=self.auth_managers)
                 fb.register()
                 fb.apply(rollback)
                 fb.set('master', self['master'])
