@@ -367,6 +367,11 @@ class NFCTAttr(object):
 
 
 class NFCTAttrTuple(NFCTAttr):
+
+    __slots__ = ('saddr', 'daddr', 'proto', 'sport', 'dport',
+                 'icmp_id', 'icmp_type', 'family', '_attr_ip',
+                 '_attr_icmp')
+
     def __init__(self, family=socket.AF_INET,
                  saddr=None, daddr=None, proto=None, sport=None, dport=None,
                  icmp_id=None, icmp_type=None, icmp_code=None):
@@ -587,8 +592,8 @@ class NFCTSocket(NetlinkSocket):
         IPCTNL_MSG_CT_GET_UNCONFIRMED: nfct_msg,
     }.items())
 
-    def __init__(self, nfgen_family=socket.AF_INET):
-        super(NFCTSocket, self).__init__(family=NETLINK_NETFILTER)
+    def __init__(self, nfgen_family=socket.AF_INET, **kwargs):
+        super(NFCTSocket, self).__init__(family=NETLINK_NETFILTER, **kwargs)
         self.register_policy(self.policy)
         self._nfgen_family = nfgen_family
 
